Imports System
Imports System.Text
Imports System.ComponentModel
Imports CodeSmith.Engine
Imports System.Data
Imports SchemaExplorer

Public Class SqlCodeTemplate

   Inherits CodeTemplate
   'This variable is used to control the indent of the 
   'formatted code that is output in conjunction with
   'the Level method
   Const INDENT_LEVEL_SPACES As Integer = 3


    Public Enum AccessibilityEnum
        [Public]
        [Protected]
        [Friend]
        [Private]
    End Enum
    Public Function GetVBAccessModifier(ByVal Accessibility As AccessibilityEnum) As String
        Select Case (Accessibility)
            Case AccessibilityEnum.Public
                Return "Public"
            Case AccessibilityEnum.Protected
                Return "Protected"
            Case AccessibilityEnum.Friend
                Return "Friend"
            Case AccessibilityEnum.Private
                Return "Private"
            Case Else
                Return "Public"
        End Select
    End Function

    Public Function GetClassName(ByVal Table As TableSchema) As String

        Static oDic As System.Collections.Specialized.StringDictionary = New System.Collections.Specialized.StringDictionary

        Dim sTableName As String
        Dim sClassName As String

        sClassName = oDic.Item(Table.Name)
        If Not sClassName Is Nothing Then
            Return sClassName
        End If
        sTableName = Table.Name
        Try
            sClassName = ""
            sClassName = GetDescriptionProperty("Single", Table.PrimaryKey.MemberColumns(0))
        Catch
        End Try
        If sClassName <> "" Then
            oDic.Add(Table.Name, sClassName)
            Return sClassName
        End If
        If (Table.Name.EndsWith("n")) Then ' use this in german
            sClassName = Table.Name.Substring(0, Table.Name.Length - 1)
        Else
            sClassName = Table.Name
        End If
        sClassName = InputBox("Singular name for Table " & Table.Name, , sClassName)
        oDic.Add(Table.Name, sClassName)
        Return sClassName
    End Function

    Public Function GetVBVariableType(ByVal Column As ColumnSchema) As String

        Select Case (Column.DataType)
            Case DbType.AnsiString
                Return "String"
            Case DbType.AnsiStringFixedLength
                Return "String"
            Case DbType.Binary
                Return "Byte()"
            Case DbType.Boolean
                Return "Boolean"
            Case DbType.Byte
                Return "Byte"
            Case DbType.Currency
                Return "Decimal"
            Case DbType.Date
                Return "Date"
            Case DbType.DateTime
                Return "Date"
            Case DbType.Decimal
                Return "Decimal"
            Case DbType.Double
                Return "Double"
            Case DbType.Guid
                Return "Guid"
            Case DbType.Int16
                Return "Short"
            Case DbType.Int32
                Return "Integer"
            Case DbType.Int64
                Return "Long"
            Case DbType.Object
                Return "object"
            Case DbType.SByte
                Return "Sbyte!!!!"
            Case DbType.Single
                Return "Float"
            Case DbType.String
                Return "String"
            Case DbType.StringFixedLength
                Return "String"
            Case DbType.Time
                Return "TimeSpan"
            Case DbType.UInt16
                Return "Ushort!!!"
            Case DbType.UInt32
                Return "Uint!!!"
            Case DbType.UInt64
                Return "Ulong!!!!"
            Case DbType.VarNumeric
                Return "Decimal"
            Case Else
                Return "__UNKNOWN__" + Column.NativeType
        End Select
    End Function
    Public Function GetVBXPOStringSizeAttribute(ByVal Column As ColumnSchema) As String
        Dim s As String
        s = GetVBVariableType(Column)
        If s = "String" Then
            If Column.Size >= 1073741823 Then
                Return "Size(SizeAttribute.Unlimited)"
            Else
                Return "Size(" & Column.Size.ToString & ")"
            End If
        End If
    End Function
    Public Function GetVBVariablePrefix(ByVal Infix As String, ByVal Column As ColumnSchema) As String
        If (Column.Name.EndsWith("TypeCode")) Then Return Column.Name
        Select Case (Column.DataType)
            Case DbType.AnsiString
                Return "s" & Infix
            Case DbType.AnsiStringFixedLength
                Return "s" & Infix
            Case DbType.Binary
                Return "byte" & Infix
            Case DbType.Boolean
                Return "b" & Infix
            Case DbType.Byte
                Return "byte" & Infix
            Case DbType.Currency
                Return "y" & Infix
            Case DbType.Date
                Return "a" & Infix
            Case DbType.DateTime
                Return "a" & Infix
            Case DbType.Decimal
                Return "y" & Infix
            Case DbType.Double
                Return "d" & Infix
            Case DbType.Guid
                Return "guid" & Infix
            Case DbType.Int16
                Return "is" & Infix
            Case DbType.Int32
                Return "i" & Infix
            Case DbType.Int64
                Return "l" & Infix
            Case DbType.Object
                Return "o" & Infix
            Case DbType.SByte
                Return "Sbyte!!!!" & Infix
            Case DbType.Single
                Return "f" & Infix
            Case DbType.String
                Return "s" & Infix
            Case DbType.StringFixedLength
                Return "s" & Infix
            Case DbType.Time
                Return "TimeSpan" & Infix
            Case DbType.UInt16
                Return "Ushort!!!" & Infix
            Case DbType.UInt32
                Return "Uint!!!" & Infix
            Case DbType.UInt64
                Return "Ulong!!!!" & Infix
            Case DbType.VarNumeric
                Return "y" & Infix
            Case Else
                Return "__UNKNOWN__" + Column.NativeType
        End Select
    End Function

    Public Function GetVBVariable(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal Column As ColumnSchema) As String
        Dim sb As New StringBuilder
        sb.Append(GetVBVariablePrefix(Infix, Column))
        sb.Append(Column.Name)
        Return sb.ToString
    End Function
    Public Function GetVBVariableDeclaration(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal Column As ColumnSchema) As String
        Dim sb As New StringBuilder
        sb.Append(GetVBAccessModifier(Accessibility))
        sb.Append(" ")
        sb.Append(GetVBVariable(Accessibility, Infix, Column))
        sb.Append(" as ")
        sb.Append(GetVBVariableType(Column))
        Return sb.ToString
    End Function
    Public Function GetVbForeignKeyVariableDeclaration(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal KeyTable As TableKeySchema) As String
        Dim sb As New StringBuilder
        sb.Append(GetVBAccessModifier(Accessibility))
        sb.Append(" ")
        sb.Append("o" & Infix)
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        sb.Append(" as ")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        Return sb.ToString
    End Function
    Public Function GetVBPropertyGetSet(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal Column As ColumnSchema) As String
        Dim sb As New StringBuilder
        sb.Append(GetVBAccessModifier(Accessibility))
        sb.Append(" Property ")
        sb.Append(Column.Name & " ()")
        sb.Append(" as ")
        sb.Append(GetVBVariableType(Column))
        sb.Append(vbCrLf)
        sb.Append(vbTab & "Get" & vbCrLf)
        sb.Append(vbTab & vbTab & "return " & GetVBVariable(Accessibility, Infix, Column) & vbCrLf)
        sb.Append(vbTab & "End Get" & vbCrLf)

        sb.Append(vbTab & "Set(ByVal Value as " & GetVBVariableType(Column) & ")" & vbCrLf)
        sb.Append(vbTab & vbTab & GetVBVariable(Accessibility, Infix, Column) & " = Value" & vbCrLf)
        sb.Append(vbTab & "End Set" & vbCrLf)
        sb.Append("End Property" & vbCrLf)
        Return sb.ToString
    End Function
    Public Function GetVbForeignKeyObject(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal KeyTable As TableKeySchema) As String
        Dim sb As New StringBuilder
        sb.Append("<")
        sb.Append("Persistent(" & """" & KeyTable.ForeignKeyMemberColumns(0).Name & """" & "), _" & vbCrLf)
        sb.Append("Association(" & """")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table) & "_" & KeyTable.ForeignKeyTable.Name)
        sb.Append("""")
        sb.Append(")> _")
        sb.Append(vbCrLf)
        sb.Append(GetVBAccessModifier(Accessibility))

        sb.Append(" Property ")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table) & " ()")
        sb.Append(" as ")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        sb.Append(vbCrLf)
        sb.Append(vbTab & "Get" & vbCrLf)
        sb.Append(vbTab & vbTab & "return ")
        sb.Append("o" & Infix)
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table) & vbCrLf)
        sb.Append(vbTab & "End Get" & vbCrLf)

        sb.Append(vbTab & "Set(ByVal Value as " & GetClassName(KeyTable.PrimaryKey.Table) & ")" & vbCrLf)
        sb.Append(vbTab & vbTab & "o" & Infix & GetClassName(KeyTable.PrimaryKey.Table) & "  = Value" & vbCrLf)
        sb.Append(vbTab & "End Set" & vbCrLf)
        sb.Append("End Property" & vbCrLf)
        Return sb.ToString
    End Function
    Public Function GetVbOneSideKeyObject(ByVal Accessibility As AccessibilityEnum, ByVal Infix As String, ByVal Table As TableSchema, ByVal KeyTable As TableKeySchema) As String
        Dim sb As New StringBuilder
        If Table.Name = KeyTable.ForeignKeyTable.Name Then
            Return ""
        End If
        sb.Append("<Aggregated(), Association(" & """")
        sb.Append(GetClassName(Table) & "_" & KeyTable.ForeignKeyTable.Name)
        sb.Append("""" & " , Gettype(" & GetClassName(KeyTable.ForeignKeyTable) & "))> _ ")
        sb.Append(vbCrLf)
        sb.Append(GetVBAccessModifier(Accessibility))
        sb.Append(" Readonly Property ")
        sb.Append(KeyTable.ForeignKeyTable.Name & " () as XPCollection")
        sb.Append(vbCrLf)
        sb.Append(vbTab & "Get" & vbCrLf)
        sb.Append(vbTab & "Return GetCollection(")
        sb.Append("""" & KeyTable.ForeignKeyTable.Name & """" & ")" & vbCrLf)
        sb.Append(vbTab & "End Get" & vbCrLf)
        sb.Append("End Property" & vbCrLf)
        Return sb.ToString
    End Function
    Public Function GetVbTableKeyDescription(ByVal Table As TableSchema) As String
        Dim sb As New StringBuilder
        Dim k As TableKeySchema
        Dim pk As PrimaryKeySchema
        Dim c As ColumnSchema
        sb.Append(String.Format("' Key overview for table {0} in database {1}", Table.Name, Table.Database.Name) & vbCrLf)
        sb.Append("' ========================================================" & vbCrLf & vbCrLf)
        sb.Append("' Primary Key" & vbCrLf)
        sb.Append("' ------------" & vbCrLf)

        For Each c In Table.PrimaryKey.MemberColumns
            sb.Append(String.Format("' {0}Column: {1}", vbTab, c.Name) & vbCrLf)
        Next

        sb.Append(vbCrLf)
        sb.Append("' Keys  " & vbCrLf)
        sb.Append("' ------" & vbCrLf)
        For Each k In Table.Keys
            sb.Append(String.Format("' Key: {0} PrimaryKeytable: {1}", k.Name, k.PrimaryKeyTable.Name & vbCrLf))
            sb.Append(String.Format("' {0} ForeignKeytable: {1}", vbTab, k.ForeignKeyTable.Name) & vbCrLf)

            For Each c In k.ForeignKeyMemberColumns
                sb.Append(String.Format("' {0}Column: {1}", vbTab, c.Name) & vbCrLf)
            Next
        Next
        sb.Append(vbCrLf)
        sb.Append("' Foreign Keys  " & vbCrLf)
        sb.Append("' --------------" & vbCrLf)
        For Each k In Table.ForeignKeys
            sb.Append(String.Format("' Key: {0} PrimaryKeytable: {1}", k.Name, k.PrimaryKeyTable.Name & vbCrLf))
            sb.Append(String.Format("' {0} ForeignKeytable: {1}", vbTab, k.ForeignKeyTable.Name) & vbCrLf)

            For Each c In k.ForeignKeyMemberColumns
                sb.Append(String.Format("' {0}Column: {1}", vbTab, c.Name) & vbCrLf)
            Next
        Next

        Return sb.ToString
    End Function
    Public Function Level(ByVal n As Integer, _
                          Optional ByVal NewLine As Boolean = False) As String
        'Provides formatting for the outputted code
        'The INDENT_LEVEL_SPACES defines how many spaces 
        '  are in a level and is declared at the top of the class
        Dim sLevel As String

        If Not NewLine Then
            sLevel = Space((n * INDENT_LEVEL_SPACES))

        Else
            sLevel = vbNewLine + Space((n * INDENT_LEVEL_SPACES))
        End If

        Return sLevel
    End Function
    Public Function GetDescriptionProperty(ByVal Key As String, ByVal Column As ColumnSchema) As String
        Dim sDescription As String
        Dim s As String
        Dim i As Integer

        Key = Trim(Key) & "="
        Try
            sDescription = Column.Description
        Catch
            Return ""
        End Try
        If sDescription = String.Empty Then Return ""
        i = sDescription.ToLower.IndexOf(Key.ToLower)
        If i = -1 Then Return ""
        s = sDescription.Substring(i + Key.Length)
        s = Strings.SysTok(s, ",", True)
        Return s
    End Function
    Public Function GetVBColumnDefault(ByVal Column As ColumnSchema) As String
        Dim sDefault As String
        Dim sType As String
        Dim i As Integer
        Dim s As String
        Try
            sDefault = CType(Column.ExtendedProperties("CS_DEFAULT").Value, String)
        Catch
            Return ""
        End Try
        If sDefault = String.Empty Then Return ""
        sType = GetVBVariableType(Column)
        Select Case sType
            Case "String"
                s = sDefault
                If s.StartsWith("(") AndAlso s.EndsWith(")") Then s = s.Substring(1, s.Length - 2)
                If s.StartsWith("'") AndAlso s.EndsWith("'") Then s = """" & s.Substring(1, s.Length - 2) & """"
            Case "Boolean"
                s = sDefault
                If s.StartsWith("(") AndAlso s.EndsWith(")") Then s = s.Substring(1, s.Length - 2)
                If s = "0" Then
                    s = "False"
                Else
                    s = "True"
                End If
            Case "Decimal"
                s = sDefault
                If s.StartsWith("(") AndAlso s.EndsWith(")") Then s = s.Substring(1, s.Length - 2)
            Case "Date"
                s = sDefault
                If s.StartsWith("(") AndAlso s.EndsWith(")") Then s = s.Substring(1, s.Length - 2)
                If s.StartsWith("'") AndAlso s.EndsWith("'") Then s = "#" & s.Substring(1, s.Length - 2) & "#"
            Case "Double", "Short", "Integer", "Long", "Float"
                s = sDefault
                If s.StartsWith("(") AndAlso s.EndsWith(")") Then s = s.Substring(1, s.Length - 2)

            Case Else
                Return "__UnknownDataTypeForDefault__" + Column.NativeType
        End Select
        Return s
    End Function
    Public Function GetVbForeignKeyObjectByKey(ByVal Infix As String, ByVal KeyTable As TableKeySchema) As String
        Dim sb As New StringBuilder
        Dim s As String
        'omDatenbank = CType(omSession.GetObjectByKey(GetType(SD.Datenbank), i), SD.Datenbank)
        'omProjekt=CType(me.session.getobjectbykey(Gettype(Projekt),1),Projekt
        ' Loads the Object from column-default
        s = GetVBColumnDefault(KeyTable.ForeignKeyMemberColumns(0))
        If s = "" Then sb.Append("'")
        sb.Append(vbTab & "o" & Infix)
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        sb.Append("=")
        sb.Append("CType(me.session.getobjectbykey(")
        sb.Append("Gettype(")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        sb.Append("),")
        sb.Append(s)
        sb.Append("),")
        sb.Append(GetClassName(KeyTable.PrimaryKey.Table))
        sb.Append(")")
        sb.Append(vbCrLf)
        Return sb.ToString
    End Function
End Class
